from django.db import models
from django.utils.safestring import mark_safe

class Code(models.Model):
    """
    Registration codes
    """
    code = models.CharField(max_length=255, unique=True)
    is_active = models.BooleanField(default=True)

    class Meta:
        app_label = 'profiles'
        ordering = ('code',)

    def __unicode__(self):
        return mark_safe(self.code)



import string
import random

from django import forms
from django.contrib import admin
from django.contrib.admin.options import csrf_protect_m
from django.core.urlresolvers import reverse
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from .models import Code


class AutoGenerateForm(forms.Form):
    number = forms.IntegerField(label=_('Number of codes to generate'))
    length = forms.IntegerField(
            label=_('Length of codes to generate'),
            initial=8,
            max_value=255,
            min_value=1)


class CodeAdmin(admin.ModelAdmin):
    list_display = ('code', 'is_active',)
    list_filter = ('is_active',)
    search_fields = ('code',)
    ordering = ['code']

    def get_urls(self):
        from django.conf.urls import patterns, include, url

        urls = super(CodeAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(r'^code-auto-generate/$',
                self.admin_site.admin_view(self.auto_generate),
                {},
                name='code_auto_generate'
            )
        )

        return my_urls + urls

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        if request.method == 'POST':
            auto_generate_form = AutoGenerateForm(request.POST)
        else:
            auto_generate_form = AutoGenerateForm()
        extra_context['auto_generate_form'] = auto_generate_form
        return super(CodeAdmin, self).changelist_view(request, extra_context=extra_context)

    @csrf_protect_m
    def auto_generate(self, request):
        if request.method == 'POST':
            if not self.has_add_permission(request):
                raise PermissionDenied
            form = AutoGenerateForm(request.POST)
            if form.is_valid():
                number = form.cleaned_data.get('number')
                length = form.cleaned_data.get('length')
                for i in range(number):
                    code = CodeAdmin.generate_code(length)
                    while True:
                        code = CodeAdmin.generate_code(length)
                        if not Code.objects.filter(code=code).count():
                            break

                    Code.objects.create(code=code, is_active=True)

            else:
                return self.changelist_view(request, extra_context={'auto_generate_form': form})

        return HttpResponseRedirect(reverse('admin:profiles_code_changelist'))

    @staticmethod
    def generate_code(length):
        """Generate a random string of ascii_uppercase case chars and digits."""
        return ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(length))


admin.site.register(Code, CodeAdmin)



from django.db import models
from django.utils.translation import ugettext_lazy as _

def register(cls, admin_cls):
    cls.add_to_class('code', models.ForeignKey('profiles.Code',
        verbose_name=_('Registration code'), null=True, blank=True))

    if admin_cls:
        admin_cls.list_display_filter += ['code', ]

        if admin_cls.fieldsets:
            admin_cls.fieldsets.append((_('Registration code'), {
                    'fields': ['code',],
                    'classes': ('collapse',),
                }))



from django.db import models
from django.utils.translation import ugettext_lazy as _

from orderable.models import Orderable
from profiles.models import Profile


class ProfileFieldsChoices(object):
    """
    Iterable that will yield Profile fields as choices. 
    This is necessary to delay accessing the Profile class until it has been defined.
    """

    def __init__(self ):
        self.count = 0

    def next(self):
        if self.count >= len(Profile._meta.fields):
            self.count = 0
            raise StopIteration
        else:
            c = self.count
            self.count += 1
            return self.__getitem__(c)

    def __getitem__(self, item):
        field = Profile._meta.fields[item]
        return (field.name, unicode(field.verbose_name))

    def __iter__(self):
        return self

class Notification(Orderable):
    """
    Patient notification methods (e.g. Email, Mobile,)
    """

    name = models.CharField(max_length=150)
    slug = models.SlugField(max_length=150)
    field_name = models.CharField(max_length=31, null=True, blank=True, 
                                  choices=ProfileFieldsChoices(),
                                  help_text=_('Name of the Profile field required for this notification.'))
   
    class Meta:
        app_label = 'profiles'
        ordering = ('sort_order',)
   
    def __unicode__(self):
        return self.name




from django.db import models
from django.utils.translation import ugettext_lazy as _

def register(cls, admin_cls):
    if admin_cls:
        admin_cls.list_display_filter += ['is_staff', 'groups', ]
        if admin_cls.fieldsets:
            admin_cls.fieldsets.append(
                (_('Permissions'), { 'fields': ('is_staff', 'groups'), 'classes': ('collapse',), })
            )




from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login, authenticate
from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse, get_callable
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView, CreateView, UpdateView

from profiles.models import Profile
from profiles.signals import user_registered, user_updated
from profiles.utils import class_view_decorator, generate_id


@class_view_decorator(login_required)
class ProfileView(TemplateView):
    template_name = 'profiles/profile.html'


class ProfleFormMixin(object):
    def get_form_class(self):
        if self.form_class:
            return self.form_class

        # Delay defining the ProfileForm to ensure that the model has been
        # defined (with all extensions).
        try:
            ProfileForm = get_callable(settings.PROFILE_FORM_CLASS)
        except AttributeError:
            from .forms import ProfileForm
        return ProfileForm


class RegisterView(ProfleFormMixin, CreateView):
    template_name = 'profiles/profile_form.html'
    success_url = getattr(settings, 'REGISTRATION_COMPLETE_URL', settings.LOGIN_REDIRECT_URL)

    def authenticate_new_user(self, username, password):
        if password:
            user = authenticate(username=username, password=password)
            messages.info(self.request, _('Your profile has been created.'))
            login(self.request, user)
        return user

    def get_form_class(self):
        if self.form_class:
            return self.form_class

        # Delay defining the form to ensure that the model has been
        # defined (with all extensions).
        try:
            RegistrationForm = get_callable(settings.REGISTRATION_FORM_CLASS)
        except AttributeError:
            RegistrationForm = super(RegisterView, self).get_form_class()

        return RegistrationForm

    def form_valid(self, form):
        self.username = self.get_username(form.cleaned_data)

        self.password = self.get_password(form.cleaned_data)

        obj = form.save(commit=False)
        obj = self.update_user_object(obj)
        obj.save()

        self.send_signals(form, obj)

        self.authenticate_new_user(self.username, self.password)

        return super(RegisterView, self).form_valid(form)

    def get_password(self, data):
        # try to get the password
        for fname in ['password', 'password1', 'password2', 'plain_password']:
            if fname in data:
                return data[fname]
        return None

    def get_username(self, data):
        if not 'username' in data:
            # auto generate a username
            generate_kwargs = {}
            for fname in ['first_name', 'last_name', 'email']:
                if fname in data:
                    generate_kwargs[fname] = data[fname]
            return generate_id(**generate_kwargs)

    def send_signals(self, form, user):
        user_registered.send(sender=self.__class__, user=user, request=self.request, form=form)

    def update_user_object(self, user):
        if self.password:
            user.set_password(self.password)
        return user


@class_view_decorator(login_required)
class ProfileEdit(ProfleFormMixin, UpdateView):
    def form_valid(self, form):
        response = super(ProfileEdit, self).form_valid(form)
        user_updated.send(sender=self.__class__, user=self.request.user, request=self.request, form=form)
        messages.info(self.request, _('Your profile has been updated.'))
        return response

    def get_context_data(self, **kwargs):
        context = super(ProfileEdit, self).get_context_data(**kwargs)
        context['site'] = Site.objects.get_current()
        return context

    def get_object(self):
        if isinstance(self.request.user, Profile):
            return self.request.user
        return self.request.user.profile

    def get_success_url(self):
        return self.request.GET.get('next', self.success_url or reverse('profile'))



from django.db import models
from django.utils.translation import ugettext_lazy as _


def register(cls, admin_cls):
    cls.add_to_class('TITLE_CHOICES', [('Dr','Dr'),
                                       ('Professor','Professor'),
                                       ('Mr','Mr'),
                                       ('Miss','Miss'),
                                       ('Mrs','Mrs'),
                                       ('Ms','Ms'),
                                      ])
    cls.add_to_class('title', models.CharField(max_length=255, verbose_name=_('title'), null=True, blank=True, choices=cls.TITLE_CHOICES))

    from profiles.models import Profile
    get_full_name = Profile.get_full_name
    def get_full_name_title(self):
        full_name = get_full_name(self)
        if self.title:
            full_name = u'%s %s' % (self.title, full_name)

        return full_name
    cls.add_to_class('get_full_name', get_full_name_title)
    

    if admin_cls:
        admin_cls.list_display_filter += ['title', ]

        if admin_cls.fieldsets:
            fields = admin_cls.fieldsets[0][1]['fields']
            try:
                at = fields.index('first_name')
            except ValueError:
                at = len(fields)
            fields.insert(at, 'title')





import re, random
import unicodedata

from django.contrib.auth.models import User
from django.utils.decorators import method_decorator

_is_alnum_re = re.compile(r'\w+')
_ID_MIN_LENGTH = 5  # minimum reasonable length for username
_ID_MAX_LENGTH = 30 # as defined in django.auth.contrib.models.User.username field


def class_view_decorator(function_decorator):
    """Convert a function based decorator into a class based decorator usable
    on class based Views.

    Follows the general idea from https://docs.djangoproject.com/en/dev/topics/class-based-views/#decorating-the-class.

    Can't subclass the `View` as it breaks inheritance (super in particular),
    so we monkey-patch instead.
    """

    def simple_decorator(View):
        View.dispatch = method_decorator(function_decorator)(View.dispatch)
        return View

    return simple_decorator


def _id_generator(first_name, last_name, email):
    def _alnum(s, glue=''):
        return glue.join(filter(len, _is_alnum_re.findall(s))).lower()
    # The way to generate id is by trying:
    #  1. username part of email
    #  2. ascii-transliterated first+last name
    #  3. whole email with non-alphanumerics replaced by underscore
    #  4. random string
    # Every try must return at least _ID_MIN_LENGTH chars to succeed and is truncated
    # to _ID_MAX_LENGTH. All IDs are lowercased.
    id = _alnum(email.split('@')[0])
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    id = _alnum(unicodedata.normalize('NFKD', unicode(first_name + last_name)).encode('ascii', 'ignore'))
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    id = _alnum(email, glue='_')
    if len(id) >= _ID_MIN_LENGTH:
        yield id[:_ID_MAX_LENGTH]
    while True:
        yield _alnum('%s_%s' % (id[:_ID_MIN_LENGTH], random_string(_ID_MIN_LENGTH, True)))[:_ID_MAX_LENGTH]


def generate_id(first_name='', last_name='', email=''):
    valid_id = False
    gen = _id_generator(first_name, last_name, email)
    test_name = gen.next()
    while valid_id is False:
        try:
            User.objects.get(username=test_name)
        except User.DoesNotExist:
            valid_id = True
        else:
            test_name = gen.next()
    return test_name


_LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"


def random_string(length, variable=False, charset=_LETTERS):
    if variable:
        length = random.randrange(1, length+1)
    return ''.join([random.choice(charset) for x in xrange(length)])



from django.core.management.base import LabelCommand, CommandError

class Command(LabelCommand):
    help = 'Add a profile for the specified user(s).'
    args = '[username1, username2, email1, ...]'

    def handle_label(self, username, **options):
        from django.contrib.auth.models import User
        from profiles.models import Profile

        if '@' in username:
            kwargs = {'email': username}
        else:
            kwargs = {'username': username}

        try:
            profile = Profile.objects.get(**kwargs)
        except Profile.DoesNotExist:
            try:
                user = User.objects.get(**kwargs)
            except User.DoesNotExist:
                raise CommandError("User '%s' does not exist" % username)

            print "Adding profile for %s" % username
            initial = dict([(f.name, getattr(user, f.name)) for f in user._meta.fields])
            profile = Profile(user_ptr=user, **initial)
            profile.save()
            for f in user._meta.many_to_many:
                getattr(profile, f.name).add(*getattr(user, f.name).all())

        else:
            raise CommandError("Profile '%s' already exists" % username)



from django.contrib import admin

from orderable.admin import OrderableAdmin

from .models import Option


class OptionOptions(OrderableAdmin):
    fields = ('name',)
    search_fields = ('name',)
    ordering = ['sort_order']

admin.site.register(Option, OptionOptions)



from django.db import models
from django.utils.translation import ugettext_lazy as _

def register(cls, admin_cls):
    cls.add_to_class('picture', models.ImageField(verbose_name=_('your picture'), upload_to='images/profiles',blank=True,null=True,))

    if admin_cls:
        if admin_cls.fieldsets:
            fields = admin_cls.fieldsets[0][1]['fields']
            fields.insert(len(fields), 'picture')






from django.contrib.auth.models import User

import factory

from .models import Profile


class UserFactory(factory.Factory):
    FACTORY_FOR = User
    first_name = factory.Sequence(lambda n: 'Firstname {0}'.format(n))
    last_name = factory.Sequence(lambda n: 'Lastname {0}'.format(n))
    username = factory.Sequence(lambda n: 'user-{0}'.format(n).lower())
    email = factory.LazyAttribute(lambda a: '{0}@example.com'.format(a.username).lower())

    @classmethod
    def _prepare(cls, create, **kwargs):
        password = kwargs.pop('password', 'password')
        user = super(UserFactory, cls)._prepare(create=False, **kwargs)
        user.set_password(password)
        user.raw_password = password
        if create:
            user.save()
        return user


class ProfileFactory(UserFactory):
    FACTORY_FOR = Profile
    user_ptr = factory.SubFactory(UserFactory)


class UserWithProfileFactory(ProfileFactory):
    """
    Return a user object that has a profile.
    """
    @classmethod
    def _prepare(cls, create, **kwargs):
        profile = super(UserWithProfileFactory, cls)._prepare(create, **kwargs)
        return profile.user_ptr


class ProfileUtils(object):
    def generate_profile(self, **kwargs):
        password = kwargs.pop('password', 'test')
        profile = ProfileFactory.build(**kwargs)
        profile.set_password(password)
        profile.save()
        return profile

    def login(self, user=None, password='test'):
        user = user or self.user
        self.client.login(username=user.username, password=password)




import re

from django.core import exceptions
from django.forms import fields

from .models import Code


class CodeField(fields.CharField):
    def to_python(self, value):
        if not value:
            return value

        try:
            value = value.strip()
            value = Code.objects.get(code__iexact=value, is_active=True)
        except Code.DoesNotExist:
            raise exceptions.ValidationError('Enter a valid registration code.')
        return value


from django.conf import settings
from django.conf.urls.defaults import *

from profiles.views import ProfileEdit, ProfileView, RegisterView


urlpatterns = patterns('',
    url(r'^$', ProfileView.as_view(), name='profile'),
    url(r'^edit/$', ProfileEdit.as_view(), name='profile-edit'),
)

if getattr(settings, 'REGISTRATION_ENABLED', False):
    urlpatterns += patterns('',
        url(r'^register/$', RegisterView.as_view(), name='profile-register'),
    )



from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext_lazy as _

from profiles.modules.addresses.models import Address


def register(cls, admin_cls):
    cls.add_to_class('addresses', models.ManyToManyField('profiles.Address', related_name='addresses', blank=True))

    if admin_cls:
        admin_cls.fieldsets += [(_('Other addresses'), {
            'classes': ['collapse',],
            'fields': ['addresses'],
        })]
        admin_cls.filter_horizontal += ('addresses',)



from django.dispatch import Signal

# A new user has registered
user_registered = Signal(providing_args=['user', 'request'])

# A user has updated the account
user_updated = Signal(providing_args=['user', 'request'])
